@@ -35,8 +35,12 @@ type Query struct {
 	// PersonalOrganizations PersonalOrganizations
 	OrganizationDetails OrganizationDetails
 	Build               Build
-	Volume              Volume
-	Domain              *Domain
+	Volume              struct {
+		App struct {
+			Name string
+		}
+	}
+	Domain *Domain
 
 	Node  interface{}
 	Nodes []interface{}
@@ -172,11 +176,6 @@ type Query struct {
 	CreateOrganization CreateOrganizationPayload
 	DeleteOrganization DeleteOrganizationPayload
 
-	CreateVolume CreateVolumePayload
-	DeleteVolume DeleteVolumePayload
-	ExtendVolume ExtendVolumePayload
-	ForkVolume   ForkVolumePayload
-
 	AddWireGuardPeer              CreatedWireGuardPeer
 	EstablishSSHKey               SSHCertificate
 	IssueCertificate              IssuedCertificate
@@ -324,12 +323,9 @@ type App struct {
 	VMSize           VMSize
 	Regions          *[]Region
 	BackupRegions    *[]Region
-	Volumes          struct {
-		Nodes []Volume
-	}
-	TaskGroupCounts []TaskGroupCount
-	ProcessGroups   []ProcessGroup
-	HealthChecks    *struct {
+	TaskGroupCounts  []TaskGroupCount
+	ProcessGroups    []ProcessGroup
+	HealthChecks     *struct {
 		Nodes []CheckState
 	}
 	PostgresAppRole *struct {
@@ -365,88 +361,6 @@ type TaskGroupCount struct {
 	Count int
 }
 
-type Snapshot struct {
-	ID        string `json:"id"`
-	Digest    string
-	Size      string
-	CreatedAt time.Time
-}
-
-type Volume struct {
-	ID  string `json:"id"`
-	App struct {
-		Name            string
-		PlatformVersion string
-	}
-	Name      string
-	SizeGb    int
-	Snapshots struct {
-		Nodes []Snapshot
-	}
-	State              string
-	Region             string
-	Encrypted          bool
-	CreatedAt          time.Time
-	AttachedAllocation *AllocationStatus
-	AttachedMachine    *GqlMachine
-	Host               struct {
-		ID string
-	}
-}
-
-func (v *Volume) IsAttached() bool {
-	return v.AttachedAllocation != nil || v.AttachedMachine != nil
-}
-
-type CreateVolumeInput struct {
-	AppID             string  `json:"appId"`
-	Name              string  `json:"name"`
-	Region            string  `json:"region"`
-	SizeGb            int     `json:"sizeGb"`
-	Encrypted         bool    `json:"encrypted"`
-	SnapshotID        *string `json:"snapshotId,omitempty"`
-	RequireUniqueZone bool    `json:"requireUniqueZone"`
-}
-
-type ExtendVolumeInput struct {
-	VolumeID string `json:"volumeId"`
-	SizeGb   int    `json:"sizeGb"`
-}
-
-type CreateVolumePayload struct {
-	App    App
-	Volume Volume
-}
-
-type ExtendVolumePayload struct {
-	App          App
-	Volume       Volume
-	NeedsRestart bool
-}
-
-type DeleteVolumeInput struct {
-	VolumeID string `json:"volumeId"`
-	LockID   string `json:"lockId,omitempty"`
-}
-
-type DeleteVolumePayload struct {
-	App App
-}
-
-type ForkVolumeInput struct {
-	AppID          string `json:"appId"`
-	SourceVolumeID string `json:"sourceVolId"`
-	Name           string `json:"name,omitempty"`
-	MachinesOnly   bool   `json:"machinesOnly,omitempty"`
-	LockID         string `json:"lockId,omitempty"`
-	Remote         bool   `json:"remote,omitempty"`
-}
-
-type ForkVolumePayload struct {
-	App    App
-	Volume Volume
-}
-
 type AppCertsCompact struct {
 	Certificates struct {
 		Nodes []AppCertificateCompact
@@ -1293,7 +1207,7 @@ type Image struct {
 	ID             string
 	Digest         string
 	Ref            string
-	CompressedSize uint64
+	CompressedSize string
 }
 
 type ReleaseCommand struct {
