@@ -12,6 +12,7 @@ import (
 	"strconv"
 	"time"
 
+	"github.com/MakeNowJust/heredoc/v2"
 	"github.com/logrusorgru/aurora"
 	"github.com/spf13/cobra"
 	"github.com/superfly/flyctl/api"
@@ -38,7 +39,7 @@ func New(usage, short, long string, fn Runner, p ...preparers.Preparer) *cobra.C
 	return &cobra.Command{
 		Use:   usage,
 		Short: short,
-		Long:  long,
+		Long:  heredoc.Doc(long),
 		RunE:  newRunE(fn, p...),
 	}
 }
@@ -346,6 +347,7 @@ func promptAndAutoUpdate(ctx context.Context) (context.Context, error) {
 	cfg := config.FromContext(ctx)
 	if shouldIgnore(ctx, [][]string{
 		{"version", "upgrade"},
+		{"settings", "autoupdate"},
 	}) {
 		return ctx, nil
 	}
@@ -391,7 +393,7 @@ func promptAndAutoUpdate(ctx context.Context) (context.Context, error) {
 
 	// The env.IsCI check is technically redundant (it should be done in update.Check), but
 	// it's nice to be extra cautious.
-	if cfg.AutoUpdate && !env.IsCI() {
+	if cfg.AutoUpdate && !env.IsCI() && update.CanUpdateThisInstallation() {
 		if versionInvalidMsg != "" || current.SeverelyOutdated(latest) {
 			if !silent {
 				fmt.Fprintln(io.ErrOut, colorize.Green(fmt.Sprintf("Automatically updating %s -> %s.", current, latestRel.Version)))
