@@ -51,6 +51,10 @@ var CommonFlags = flag.Set{
 		Name:        "provision-extensions",
 		Description: "Provision any extensions assigned as a default to first deployments",
 	},
+	flag.Bool{
+		Name:        "no-extensions",
+		Description: "Do not provision Sentry nor other auto-provisioned extensions",
+	},
 	flag.StringArray{
 		Name:        "env",
 		Shorthand:   "e",
@@ -102,6 +106,11 @@ var CommonFlags = flag.Set{
 		Description: "Perform smoke checks during deployment",
 		Default:     true,
 	},
+	flag.Float64{
+		Name:        "max-unavailable",
+		Description: "Max number of unavailable machines during rolling updates. A number between 0 and 1 means percent of total machines",
+		Default:     0.33,
+	},
 	flag.Bool{
 		Name:        "no-public-ips",
 		Description: "Do not allocate any new public IP addresses",
@@ -120,15 +129,6 @@ var CommonFlags = flag.Set{
 		Description: "Memory (in megabytes) to attribute to the VM",
 		Aliases:     []string{"memory"},
 	},
-	flag.Bool{
-		Name: "strategy-batch",
-		Description: fmt.Sprintf(
-			"For \"rolling\" and \"canary\": batch Machine updates into %d groups when updating more than %d.",
-			batchingGroupCount,
-			batchingCutoff,
-		),
-		Default: true,
-	},
 	flag.StringArray{
 		Name:        "file-local",
 		Description: "Set of files in the form of /path/inside/machine=<local/path> pairs. Can be specified multiple times.",
@@ -141,6 +141,14 @@ var CommonFlags = flag.Set{
 		Name:        "file-secret",
 		Description: "Set of secrets in the form of /path/inside/machine=SECRET pairs where SECRET is the name of the secret. Can be specified multiple times.",
 	},
+	flag.StringSlice{
+		Name:        "exclude-regions",
+		Description: "Deploy to all machines except machines in these regions. Multiple regions can be specified with comma separated values or by providing the flag multiple times. --exclude-regions iad,sea --exclude-regions syd will exclude all three iad, sea, and syd regions. Applied after --only-regions. V2 machines platform only.",
+	},
+	flag.StringSlice{
+		Name:        "only-regions",
+		Description: "Deploy to machines only in these regions. Multiple regions can be specified with comma separated values or by providing the flag multiple times. --only-regions iad,sea --only-regions syd will deploy to all three iad, sea, and syd regions. Applied before --exclude-regions. V2 machines platform only.",
+	},
 }
 
 func New() (cmd *cobra.Command) {
@@ -209,7 +217,6 @@ func DeployWithConfig(ctx context.Context, appConfig *appconfig.Config, forceYes
 				warning := fmt.Sprintf("%s %s may be a potentially sensitive environment variable. Consider setting it as a secret, and removing it from the [env] section: https://fly.io/docs/reference/secrets/\n", aurora.Yellow("WARN"), env)
 				fmt.Fprintln(io.ErrOut, warning)
 			}
-
 		}
 	}
 
@@ -223,7 +230,7 @@ func DeployWithConfig(ctx context.Context, appConfig *appconfig.Config, forceYes
 		return nil
 	}
 
-	fmt.Fprintf(io.Out, "\nWatch your app at https://fly.io/apps/%s/monitoring\n\n", appName)
+	fmt.Fprintf(io.Out, "\nWatch your deployment at https://fly.io/apps/%s/monitoring\n\n", appName)
 	if useMachines(ctx, appCompact) {
 		if err := appConfig.EnsureV2Config(); err != nil {
 			return fmt.Errorf("Can't deploy an invalid v2 app config: %s", err)
@@ -321,24 +328,41 @@ func deployToMachines(
 		_ = ApplyFlagsToGuest(ctx, guest)
 	}
 
+	excludeRegions := make(map[string]interface{})
+	for _, r := range flag.GetStringSlice(ctx, "exclude-regions") {
+		reg := strings.TrimSpace(r)
+		if reg != "" {
+			excludeRegions[reg] = struct{}{}
+		}
+	}
+	onlyRegions := make(map[string]interface{})
+	for _, r := range flag.GetStringSlice(ctx, "only-regions") {
+		reg := strings.TrimSpace(r)
+		if reg != "" {
+			onlyRegions[reg] = struct{}{}
+		}
+	}
+
 	md, err := NewMachineDeployment(ctx, MachineDeploymentArgs{
 		AppCompact:            appCompact,
 		DeploymentImage:       img.Tag,
 		Strategy:              flag.GetString(ctx, "strategy"),
-		BatchMachineWaits:     flag.GetBool(ctx, "strategy-batch"),
 		EnvFromFlags:          flag.GetStringArray(ctx, "env"),
 		PrimaryRegionFlag:     appConfig.PrimaryRegion,
 		SkipSmokeChecks:       flag.GetDetach(ctx) || !flag.GetBool(ctx, "smoke-checks"),
 		SkipHealthChecks:      flag.GetDetach(ctx),
 		WaitTimeout:           time.Duration(flag.GetInt(ctx, "wait-timeout")) * time.Second,
 		LeaseTimeout:          time.Duration(flag.GetInt(ctx, "lease-timeout")) * time.Second,
+		MaxUnavailable:        flag.GetFloat64(ctx, "max-unavailable"),
 		ReleaseCmdTimeout:     releaseCmdTimeout,
 		Guest:                 guest,
 		IncreasedAvailability: flag.GetBool(ctx, "ha"),
 		AllocPublicIP:         !flag.GetBool(ctx, "no-public-ips"),
 		UpdateOnly:            flag.GetBool(ctx, "update-only"),
 		Files:                 files,
-		ProvisionExtensions:   flag.GetBool(ctx, "provision-extensions"),
+		ExcludeRegions:        excludeRegions,
+		NoExtensions:          flag.GetBool(ctx, "no-extensions"),
+		OnlyRegions:           onlyRegions,
 	})
 	if err != nil {
 		sentry.CaptureExceptionWithAppInfo(err, "deploy", appCompact)
@@ -440,6 +464,7 @@ func determineAppConfig(ctx context.Context) (cfg *appconfig.Config, err error)
 		cfg.SetEnvVariables(parsedEnv)
 	}
 
+	// FIXME: this is a confusing flag; I thought it meant only update machines in the provided region, which resulted in a minor disaster :-)
 	if v := flag.GetRegion(ctx); v != "" {
 		cfg.PrimaryRegion = v
 	}
