@@ -6,8 +6,8 @@ import (
 
 	"github.com/samber/lo"
 	"github.com/superfly/flyctl/api"
+	"github.com/superfly/flyctl/gql"
 	extensions_core "github.com/superfly/flyctl/internal/command/extensions/core"
-	"github.com/superfly/flyctl/internal/env"
 	"github.com/superfly/flyctl/internal/prompt"
 )
 
@@ -22,22 +22,16 @@ func (md *machineDeployment) provisionFirstDeploy(ctx context.Context, allocPubl
 		return fmt.Errorf("failed to provision seed volumes: %w", err)
 	}
 
-	// Provision Sentry on first deployment, unless we're using CI, where it's more likely to see ephemeral applications
-	if !env.IsCI() && md.provisionExtensions {
-		if err := md.provisionSentryOnFirstDeploy(ctx); err != nil {
-			fmt.Fprintf(md.io.ErrOut, "Failed to provision a Sentry project for this app. Use `fly ext sentry create` to try again. ERROR: %s", err)
-			return nil
+	// Provision Sentry on first deployment unless explicitly prevented by the --no-extensions option
+	if !md.noExtensions {
+		_, err := extensions_core.ProvisionExtension(ctx, md.app.Name, "sentry", true, gql.AddOnOptions{})
+		if err != nil {
+			fmt.Fprintf(md.io.ErrOut, "Failed to provision a Sentry project for this app. Use `fly ext sentry create` to try again.\nERROR: %s", err)
 		}
 	}
-
 	return nil
 }
 
-func (md *machineDeployment) provisionSentryOnFirstDeploy(ctx context.Context) error {
-	_, err := extensions_core.ProvisionExtension(ctx, md.app.Name, extensions_core.MonitoringExtensionDefaults)
-	return err
-}
-
 func (md *machineDeployment) provisionIpsOnFirstDeploy(ctx context.Context, allocPublicIPs bool) error {
 	// Provision only if the app hasn't been deployed and have services defined
 	if !md.isFirstDeploy || len(md.appConfig.AllServices()) == 0 || !allocPublicIPs {
@@ -95,6 +89,7 @@ func (md *machineDeployment) provisionIpsOnFirstDeploy(ctx context.Context, allo
 		fmt.Fprintf(md.io.Out, "  Add a dedicated ipv4 with: fly ips allocate-v4\n")
 	}
 
+	fmt.Fprintln(md.io.Out)
 	return nil
 }
 
@@ -124,15 +119,14 @@ func (md *machineDeployment) provisionVolumesOnFirstDeploy(ctx context.Context)
 
 			fmt.Fprintf(md.io.Out, "Creating 1GB volume '%s' for process group '%s'. Use 'fly vol extend' to increase its size\n", m.Source, groupName)
 
-			input := api.CreateVolumeInput{
-				AppID:     md.app.ID,
+			input := api.CreateVolumeRequest{
 				Name:      m.Source,
 				Region:    groupConfig.PrimaryRegion,
-				SizeGb:    1,
-				Encrypted: true,
+				SizeGb:    api.Pointer(1),
+				Encrypted: api.Pointer(true),
 			}
 
-			vol, err := md.apiClient.CreateVolume(ctx, input)
+			vol, err := md.flapsClient.CreateVolume(ctx, input)
 			if err != nil {
 				return fmt.Errorf("failed creating volume: %w", err)
 			}
