@@ -27,6 +27,7 @@ const (
 	DefaultReleaseCommandTimeout = 5 * time.Minute
 	DefaultLeaseTtl              = 13 * time.Second
 	DefaultVMSize                = "shared-cpu-1x"
+	DefaultMaxUnavailable        = 0.33
 )
 
 type MachineDeployment interface {
@@ -37,11 +38,11 @@ type MachineDeploymentArgs struct {
 	AppCompact            *api.AppCompact
 	DeploymentImage       string
 	Strategy              string
-	BatchMachineWaits     bool
 	EnvFromFlags          []string
 	PrimaryRegionFlag     string
 	SkipSmokeChecks       bool
 	SkipHealthChecks      bool
+	MaxUnavailable        float64
 	RestartOnly           bool
 	WaitTimeout           time.Duration
 	LeaseTimeout          time.Duration
@@ -52,6 +53,9 @@ type MachineDeploymentArgs struct {
 	UpdateOnly            bool
 	Files                 []*api.File
 	ProvisionExtensions   bool
+	NoExtensions          bool
+	ExcludeRegions        map[string]interface{}
+	OnlyRegions           map[string]interface{}
 }
 
 type machineDeployment struct {
@@ -67,11 +71,11 @@ type machineDeployment struct {
 	releaseCommandMachine machine.MachineSet
 	volumes               map[string][]api.Volume
 	strategy              string
-	batchMachineWaits     bool
 	releaseId             string
 	releaseVersion        int
 	skipSmokeChecks       bool
 	skipHealthChecks      bool
+	maxUnavailable        float64
 	restartOnly           bool
 	waitTimeout           time.Duration
 	leaseTimeout          time.Duration
@@ -82,7 +86,9 @@ type machineDeployment struct {
 	increasedAvailability bool
 	listenAddressChecked  map[string]struct{}
 	updateOnly            bool
-	provisionExtensions   bool
+	noExtensions          bool
+	excludeRegions        map[string]interface{}
+	onlyRegions           map[string]interface{}
 }
 
 func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (MachineDeployment, error) {
@@ -133,6 +139,12 @@ func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (Mach
 	}
 	io := iostreams.FromContext(ctx)
 	apiClient := client.FromContext(ctx).API()
+
+	maxUnavailable := DefaultMaxUnavailable
+	if mu := args.MaxUnavailable; mu > 0 {
+		maxUnavailable = mu
+	}
+
 	md := &machineDeployment{
 		apiClient:             apiClient,
 		gqlClient:             apiClient.GenqClient,
@@ -142,10 +154,10 @@ func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (Mach
 		app:                   args.AppCompact,
 		appConfig:             appConfig,
 		img:                   args.DeploymentImage,
-		batchMachineWaits:     args.BatchMachineWaits,
 		skipSmokeChecks:       args.SkipSmokeChecks,
 		skipHealthChecks:      args.SkipHealthChecks,
 		restartOnly:           args.RestartOnly,
+		maxUnavailable:        maxUnavailable,
 		waitTimeout:           waitTimeout,
 		leaseTimeout:          leaseTimeout,
 		leaseDelayBetween:     leaseDelayBetween,
@@ -154,7 +166,9 @@ func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (Mach
 		listenAddressChecked:  make(map[string]struct{}),
 		updateOnly:            args.UpdateOnly,
 		machineGuest:          args.Guest,
-		provisionExtensions:   args.ProvisionExtensions,
+		noExtensions:          args.NoExtensions,
+		excludeRegions:        args.ExcludeRegions,
+		onlyRegions:           args.OnlyRegions,
 	}
 	if err := md.setStrategy(); err != nil {
 		return nil, err
@@ -202,7 +216,6 @@ func (md *machineDeployment) setMachinesForDeployment(ctx context.Context) error
 		return err
 	}
 
-	// migrate non-platform machines into fly platform
 	if len(machines) == 0 {
 		terminal.Debug("Found no machines that are part of Fly Apps Platform. Checking for active machines...")
 		activeMachines, err := md.flapsClient.ListActive(ctx)
@@ -222,6 +235,27 @@ func (md *machineDeployment) setMachinesForDeployment(ctx context.Context) error
 		}
 	}
 
+	if len(md.onlyRegions) > 0 {
+		var onlyRegionMachines []*api.Machine
+		for _, m := range machines {
+			if _, present := md.onlyRegions[m.Region]; present {
+				onlyRegionMachines = append(onlyRegionMachines, m)
+			}
+		}
+		fmt.Fprintf(md.io.ErrOut, "--only-regions filter applied, deploying to %d/%d machines\n", len(onlyRegionMachines), len(machines))
+		machines = onlyRegionMachines
+	}
+	if len(md.excludeRegions) > 0 {
+		var excludeRegionMachines []*api.Machine
+		for _, m := range machines {
+			if _, present := md.excludeRegions[m.Region]; !present {
+				excludeRegionMachines = append(excludeRegionMachines, m)
+			}
+		}
+		fmt.Fprintf(md.io.ErrOut, "--exclude-regions filter applied, deploying to %d/%d machines\n", len(excludeRegionMachines), len(machines))
+		machines = excludeRegionMachines
+	}
+
 	for _, m := range machines {
 		if m.Config != nil && m.Config.Metadata != nil {
 			m.Config.Metadata[api.MachineConfigMetadataKeyFlyctlVersion] = buildinfo.ParsedVersion().String()
@@ -245,7 +279,7 @@ func (md *machineDeployment) setVolumes(ctx context.Context) error {
 		return nil
 	}
 
-	volumes, err := md.apiClient.GetVolumes(ctx, md.app.Name)
+	volumes, err := md.flapsClient.GetVolumes(ctx)
 	if err != nil {
 		return fmt.Errorf("Error fetching application volumes: %w", err)
 	}
