@@ -92,11 +92,6 @@ func newMigrateToV2() *cobra.Command {
 			Default:     false,
 			Hidden:      true,
 		},
-		flag.Bool{
-			Name:        "remote-fork",
-			Description: "PLACEHOLDER - this is the default now",
-			Hidden:      true,
-		},
 		flag.Bool{
 			Name:        "use-local-config",
 			Description: "Use local fly.toml. Do not attempt to remotely fetch the app configuration from the latest deployed release",
@@ -357,7 +352,10 @@ func NewV2PlatformMigrator(ctx context.Context, appName string) (V2PlatformMigra
 	if err != nil {
 		return nil, err
 	}
-	migrator.resolveOldVolumes()
+	err = migrator.resolveOldVolumes(ctx)
+	if err != nil {
+		return nil, err
+	}
 	err = migrator.validateVolumes(ctx)
 	if err != nil {
 		return nil, err
@@ -411,7 +409,7 @@ func (m *v2PlatformMigrator) rollback(ctx context.Context, tb *render.TextBlock)
 		for _, vol := range m.createdVolumes {
 			if err := retry.Do(
 				func() error {
-					_, err := m.apiClient.DeleteVolume(ctx, vol.vol.ID, m.appLock)
+					_, err := m.flapsClient.DeleteVolume(ctx, vol.vol.ID)
 					return err
 				},
 				retry.Context(ctx), retry.Attempts(10),
