@@ -77,10 +77,9 @@ func (m *v2PlatformMigrator) updateNomadPostgresImage(ctx context.Context) error
 }
 
 func (m *v2PlatformMigrator) migratePgVolumes(ctx context.Context) error {
-	app := m.appFull
 	regionsToVols := map[string][]api.Volume{}
 	// Find all volumes
-	for _, vol := range app.Volumes.Nodes {
+	for _, vol := range m.oldAttachedVolumes {
 		if strings.Contains(vol.Name, "machines") || vol.AttachedAllocation == nil {
 			continue
 		}
@@ -91,21 +90,20 @@ func (m *v2PlatformMigrator) migratePgVolumes(ctx context.Context) error {
 	for region, vols := range regionsToVols {
 		fmt.Fprintf(m.io.Out, "Creatings %d new volume(s) in '%s'\n", len(vols), region)
 		for _, vol := range vols {
-			// TODO: make use of https://github.com/superfly/nomad-firecracker/pull/1013
-			input := api.CreateVolumeInput{
-				AppID:     app.ID,
-				Name:      fmt.Sprintf("%s_machines", vol.Name),
-				Region:    region,
-				SizeGb:    vol.SizeGb,
-				Encrypted: vol.Encrypted,
+			input := api.CreateVolumeRequest{
+				Name:         fmt.Sprintf("%s_machines", vol.Name),
+				Region:       region,
+				SizeGb:       &vol.SizeGb,
+				Encrypted:    api.Pointer(vol.Encrypted),
+				MachinesOnly: api.Pointer(true),
 			}
-			newVol, err := m.apiClient.CreateVolume(ctx, input)
+			newVol, err := m.flapsClient.CreateVolume(ctx, input)
 			if err != nil {
 				return err
 			}
 			newVols = append(newVols, &NewVolume{
 				vol:             newVol,
-				previousAllocId: vol.AttachedAllocation.ID,
+				previousAllocId: *vol.AttachedAllocation,
 				mountPoint:      "/data",
 			})
 		}
