@@ -61,13 +61,10 @@ func (m *v2PlatformMigrator) migrateAppVolumes(ctx context.Context) error {
 	}))
 
 	for _, vol := range m.oldAttachedVolumes {
-		newVol, err := m.apiClient.ForkVolume(ctx, api.ForkVolumeInput{
-			AppID:          m.appFull.ID,
-			SourceVolumeID: vol.ID,
-			MachinesOnly:   true,
-			Remote:         true,
+		newVol, err := m.flapsClient.CreateVolume(ctx, api.CreateVolumeRequest{
+			SourceVolumeID: &vol.ID,
+			MachinesOnly:   api.Pointer(true),
 			Name:           vol.Name,
-			LockID:         m.appLock,
 		})
 		if err != nil && strings.HasSuffix(err.Error(), " is not a valid candidate") {
 			return fmt.Errorf("unfortunately the worker hosting your volume %s (%s) does not have capacity for another volume to support the migration; some other options: 1) try again later and there might be more space on the worker, 2) run a manual migration https://community.fly.io/t/manual-migration-to-apps-v2/11870, or 3) wait until we support volume migrations across workers (we're working on it!)", vol.ID, vol.Name)
@@ -77,17 +74,16 @@ func (m *v2PlatformMigrator) migrateAppVolumes(ctx context.Context) error {
 
 		allocId := ""
 		path := ""
-		if alloc := vol.AttachedAllocation; alloc != nil {
-			allocId = alloc.ID
+		if allocId := vol.AttachedAllocation; allocId != nil {
 			alloc, ok := lo.Find(m.oldAllocs, func(a *api.AllocationStatus) bool {
-				return a.ID == allocId
+				return a.ID == *allocId
 			})
 			if !ok {
-				return fmt.Errorf("volume %s[%s] is attached to alloc %s, but that alloc is not running", vol.Name, vol.ID, allocId)
+				return fmt.Errorf("volume %s[%s] is attached to alloc %s, but that alloc is not running", vol.Name, vol.ID, *allocId)
 			}
 			path = m.nomadVolPath(&vol, alloc.TaskName)
 			if path == "" {
-				return fmt.Errorf("volume %s[%s] is mounted on alloc %s, but has no mountpoint", vol.Name, vol.ID, allocId)
+				return fmt.Errorf("volume %s[%s] is mounted on alloc %s, but has no mountpoint", vol.Name, vol.ID, *allocId)
 			}
 		}
 		if m.verbose {
@@ -116,17 +112,22 @@ func (m *v2PlatformMigrator) nomadVolPath(v *api.Volume, group string) string {
 }
 
 // Must run *after* allocs are filtered
-func (m *v2PlatformMigrator) resolveOldVolumes() {
-	m.oldAttachedVolumes = lo.Filter(m.appFull.Volumes.Nodes, func(v api.Volume, _ int) bool {
+func (m *v2PlatformMigrator) resolveOldVolumes(ctx context.Context) error {
+	vols, err := m.flapsClient.GetAllVolumes(ctx)
+	if err != nil {
+		return err
+	}
+	m.oldAttachedVolumes = lo.Filter(vols, func(v api.Volume, _ int) bool {
 		if v.AttachedAllocation != nil {
 			for _, a := range m.oldAllocs {
-				if a.ID == v.AttachedAllocation.ID {
+				if a.ID == *v.AttachedAllocation {
 					return true
 				}
 			}
 		}
 		return false
 	})
+	return nil
 }
 
 func (m *v2PlatformMigrator) printReplacedVolumes() {
