@@ -8,6 +8,7 @@ import (
 	"github.com/spf13/cobra"
 	"github.com/superfly/flyctl/api"
 	"github.com/superfly/flyctl/client"
+	"github.com/superfly/flyctl/flaps"
 	"github.com/superfly/flyctl/flypg"
 	"github.com/superfly/flyctl/internal/command"
 	"github.com/superfly/flyctl/internal/command/apps"
@@ -191,14 +192,21 @@ func run(ctx context.Context) (err error) {
 			params.ForkFrom = volID
 		}
 
+		flapsClient := flaps.FromContext(ctx)
+
 		// Resolve the volume
-		vol, err := client.GetVolume(ctx, params.ForkFrom)
+		vol, err := flapsClient.GetVolume(ctx, params.ForkFrom)
 		if err != nil {
 			return fmt.Errorf("Failed to resolve the specified fork-from volume %s: %w", params.ForkFrom, err)
 		}
 
+		appName, err := client.GetAppNameFromVolume(ctx, vol.ID)
+		if err != nil {
+			return err
+		}
+
 		// Confirm that the volume is associated with the fork-from app
-		if vol.App.Name != forkApp.Name {
+		if *appName != forkApp.Name {
 			return fmt.Errorf("The volume %q specified must be associated with the fork-from app %q", vol.ID, forkApp.Name)
 		}
 
@@ -280,14 +288,18 @@ func CreateCluster(ctx context.Context, org *api.Organization, region *api.Regio
 		fmt.Fprintf(io.Out, "For pricing information visit: https://fly.io/docs/about/pricing/#postgresql-clusters")
 
 		msg := "Select configuration:"
+		configurations := postgresConfigurations(input.Manager)
 		var selected int
 
 		options := []string{}
-		for _, cfg := range postgresConfigurations(input.Manager) {
+		for i, cfg := range configurations {
 			options = append(options, cfg.Description)
+			if selected == 0 && !strings.HasPrefix(cfg.Description, "Dev") {
+				selected = i
+			}
 		}
 
-		if err := prompt.Select(ctx, &selected, msg, "", options...); err != nil {
+		if err := prompt.Select(ctx, &selected, msg, configurations[selected].Description, options...); err != nil {
 			return err
 		}
 		config = &postgresConfigurations(input.Manager)[selected]
@@ -378,7 +390,7 @@ func resolveVMSize(ctx context.Context, targetSize string) (*api.VMSize, error)
 			}
 		}
 
-		return nil, fmt.Errorf("vm size %q is not valid", targetSize)
+		return nil, fmt.Errorf("VM size %q is not valid. For a full list of supported sizes use the command 'flyctl platform vm-sizes'", targetSize)
 	}
 	// prompt user to select machine specific size.
 	return prompt.SelectVMSize(ctx, MachineVMSizes())
