@@ -13,6 +13,9 @@ const (
 	// LocalOnly denotes the name of the local-only flag.
 	LocalOnly = "local-only"
 
+	// Debug denotes the name of the debug flag.
+	Debug = "debug"
+
 	// Org denotes the name of the org flag.
 	Org = "org"
 
