@@ -595,3 +595,86 @@ func TestLaunchCpusMem(t *testing.T) {
 	require.Equal(f, 8192, firstMachineGuest.MemoryMB)
 	require.Equal(f, "performance", firstMachineGuest.CPUKind)
 }
+
+func TestLaunchDetach(t *testing.T) {
+	t.Parallel()
+
+	var (
+		f       = testlib.NewTestEnvFromEnv(t)
+		appName = f.CreateRandomAppName()
+	)
+
+	res := f.Fly("launch --org %s --name %s --region %s --now --internal-port 80 --image nginx --auto-confirm --detach", f.OrgSlug(), appName, f.PrimaryRegion())
+	require.NotContains(f, res.StdErr().String(), "success")
+
+	res = f.Fly("apps destroy --yes %s", appName)
+
+	res = f.Fly("launch --org %s --name %s --region %s --now --internal-port 80 --image nginx --auto-confirm --copy-config", f.OrgSlug(), appName, f.PrimaryRegion())
+	require.Contains(f, res.StdErr().String(), "success")
+}
+
+func TestDeployDetach(t *testing.T) {
+	t.Parallel()
+
+	var (
+		f       = testlib.NewTestEnvFromEnv(t)
+		appName = f.CreateRandomAppName()
+	)
+
+	f.Fly("launch --org %s --name %s --region %s --now --internal-port 80 --image nginx --auto-confirm", f.OrgSlug(), appName, f.PrimaryRegion())
+
+	res := f.Fly("deploy --detach")
+	require.NotContains(f, res.StdErr().String(), "started")
+
+	res = f.Fly("deploy")
+	require.Contains(f, res.StdErr().String(), "started")
+}
+
+func TestDeployDetachBatching(t *testing.T) {
+	t.Parallel()
+
+	var (
+		f       = testlib.NewTestEnvFromEnv(t)
+		appName = f.CreateRandomAppName()
+	)
+
+	f.Fly("launch --org %s --name %s --region %s --now --internal-port 80 --image nginx --auto-confirm", f.OrgSlug(), appName, f.PrimaryRegion())
+	f.Fly("scale count 6 --yes")
+
+	res := f.Fly("deploy --detach")
+	require.NotContains(f, res.StdErr().String(), "started", false)
+
+	res = f.Fly("deploy")
+	require.Contains(f, res.StdErr().String(), "started", false)
+}
+
+func TestErrOutput(t *testing.T) {
+	t.Parallel()
+
+	var (
+		f       = testlib.NewTestEnvFromEnv(t)
+		appName = f.CreateRandomAppName()
+		res     *testlib.FlyctlResult
+	)
+
+	f.Fly("launch --org %s --name %s --region %s --now --internal-port 80 --image nginx --auto-confirm", f.OrgSlug(), appName, f.PrimaryRegion())
+	machList := f.MachinesList(appName)
+	firstMachine := machList[0]
+
+	res = f.FlyAllowExitFailure("machine update --vm-cpus 3 %s --yes", firstMachine.ID)
+	require.Contains(f, res.StdErr().String(), "invalid number of CPUs")
+
+	res = f.FlyAllowExitFailure("machine update --vm-memory 10 %s --yes", firstMachine.ID)
+	require.Contains(f, res.StdErr().String(), "invalid memory size")
+
+	f.Fly("machine update --vm-cpus 4 %s --vm-memory 2048 --yes", firstMachine.ID)
+
+	res = f.FlyAllowExitFailure("machine update --vm-memory 256 %s --yes", firstMachine.ID)
+	require.Contains(f, res.StdErr().String(), "memory size for config is too low")
+
+	res = f.FlyAllowExitFailure("machine update --vm-memory 16384 %s --yes", firstMachine.ID)
+	require.Contains(f, res.StdErr().String(), "memory size for config is too high")
+
+	res = f.FlyAllowExitFailure("machine update -a %s %s -y --wait-timeout 1 --vm-size performance-1x", appName, firstMachine.ID)
+	require.Contains(f, res.StdErr().String(), "timeout reached waiting for machine's state to change")
+}
