@@ -152,6 +152,10 @@ func TestFlyMachineRun_port(t *testing.T) {
 		}},
 	}}
 	require.Equal(f, want, m.Config.Services)
+	require.Contains(f, []string{"created", "started"}, m.State)
+
+	result := f.Fly("dig %s.internal -a %s --short", appName, appName)
+	require.NotEmpty(f, result.StdOut().String())
 
 	f.Fly("machine update -a %s %s -y --port 80/tcp:http --port 1001/udp", appName, m.ID)
 	m = f.MachinesList(appName)[0]
