@@ -15,6 +15,7 @@ import (
 
 	"github.com/BurntSushi/toml"
 	"github.com/google/shlex"
+	"github.com/oklog/ulid/v2"
 	"github.com/superfly/flyctl/api"
 	"github.com/superfly/flyctl/iostreams"
 )
@@ -31,6 +32,7 @@ type FlyctlTestEnv struct {
 	env                 map[string]string
 	cmdHistory          []*FlyctlResult
 	noHistoryOnFail     bool
+	id                  string
 }
 
 func (f *FlyctlTestEnv) OrgSlug() string {
@@ -41,6 +43,10 @@ func (f *FlyctlTestEnv) WorkDir() string {
 	return f.workDir
 }
 
+func (f *FlyctlTestEnv) ID() string {
+	return f.id
+}
+
 func (f *FlyctlTestEnv) PrimaryRegion() string {
 	return f.primaryRegion
 }
@@ -87,6 +93,8 @@ func NewTestEnvFromEnv(t testing.TB) *FlyctlTestEnv {
 		noHistoryOnFail: noHistoryOnFail,
 		envVariables:    make(map[string]string),
 	})
+
+	fmt.Println("workdir", env.workDir)
 	return env
 }
 
@@ -127,6 +135,7 @@ func NewTestEnvFromConfig(t testing.TB, cfg TestEnvConfig) *FlyctlTestEnv {
 		primaryReg = defaultRegion
 	}
 	testEnv := &FlyctlTestEnv{
+		id:                  ulid.Make().String(),
 		t:                   t,
 		flyctlBin:           flyctlBin,
 		primaryRegion:       primaryReg,
